﻿Imports System.Drawing.Imaging
Imports System.IO
Imports System.Net.Mime
Imports System.Security.AccessControl
Imports System.Text
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.Jobs
Imports WebMatrix.Data

Namespace ImageUploadAndProcessingJob
	Friend Class Program

		Private Shared ReadOnly Property ImageFolderPath As String
			Get
				Return Environment.ExpandEnvironmentVariables("%HOME%\site\wwwroot\App_Data\Upload")
			End Get
		End Property

        Public Shared Sub Main(ByVal args() As String)
            Dim host = New JobHost
            host.RunAndBlock()
        End Sub

		Public Shared Sub ProcessUploadQueueMessages(<QueueInput("uploadqueue")> ByVal queueMessage As String, ByVal binder As IBinder)
			Dim splited = queueMessage.Split(","c).Select(Function(m) m.Trim()).ToArray()
			Dim userName = splited(0)
			Dim galleryId = splited(1)
			Dim imageId = splited(2)
			Dim extension = splited(3)
            Dim filePath = Path.Combine(ImageFolderPath, imageId + extension)

			UploadFullImage(filePath, imageId + extension, binder)
			UploadThumbnail(filePath, imageId + extension, binder)
            SafeGuard(Sub()
                          File.Delete(filePath)
                      End Sub)

			Using db = Database.Open("PhotoGallery")
                db.Execute("INSERT INTO Photos" & _
                           ControlChars.CrLf & _
                           "                        (Id, GalleryId, UserName, Description, FileTitle, FileExtension, UploadDate, Likes) VALUES " & _
                           ControlChars.CrLf & "                        (@0, @1, @2, @3, @4, @5, @6, @7)", imageId, galleryId, userName, "", imageId, extension, Date.UtcNow, 0)
			End Using
		End Sub

		Private Shared Sub SafeGuard(ByVal action As Action)
			Try
				action.Invoke()
			Catch e As Exception
				Console.WriteLine(e)
			End Try
		End Sub

		Public Shared Sub ProcessLikeQueueMessages(<QueueInput("likequeue")> ByVal queueMessage As String)
			Dim splited = queueMessage.Split(","c).Select(Function(m) m.Trim()).ToArray()
			Dim userName = splited(0)
			Dim imageId = splited(1)

			Using db = Database.Open("PhotoGallery")
				db.Execute("UPDATE Photos SET Likes = Likes + 1 WHERE Id = @0", imageId)
				db.Execute("UPDATE GuidUsers SET TotalLikes = TotalLikes + 1 WHERE UserName = @0", userName)
			End Using
		End Sub

		Private Shared Sub UploadFullImage(ByVal imagePath As String, ByVal blobName As String, ByVal binder As IBinder)
			Using fileStream = New FileStream(imagePath, FileMode.Open)
				Using outputStream = binder.Bind(Of Stream)(New BlobOutputAttribute(String.Format("full/{0}", blobName)))
					fileStream.CopyTo(outputStream)
				End Using
			End Using
			RemoteStorageManager.ReplicateBlob("full", blobName)
		End Sub

		Private Shared Sub UploadThumbnail(ByVal imagePath As String, ByVal blobName As String, ByVal binder As IBinder)
			Using outputStream = binder.Bind(Of Stream)(New BlobOutputAttribute(String.Format("thumbnail/{0}", blobName)))
                Using imagea = Image.FromFile(imagePath)

                    Dim ratio = Min(200.0 / imagea.Width, 200.0 / imagea.Height)
                    Dim thumba = imagea.GetThumbnailImage(CInt(Fix(imagea.Width * ratio)), CInt(Fix(imagea.Height * ratio)),
                                                         Function() As Boolean
                                                             Return False
                                                         End Function, IntPtr.Zero)
                    thumba.Save(outputStream, ImageFormat.Jpeg)
                End Using
            End Using

			System.Threading.Thread.Sleep(1000)
			RemoteStorageManager.ReplicateBlob("thumbnail", blobName)
		End Sub

		Private Shared Function Min(ByVal a As Double, ByVal b As Double) As Double
			Return If(a < b, a, b)
		End Function

	End Class
End Namespace
